local BaikItemLevel = LibStub("AceAddon-3.0"):NewAddon("BaikItemLevel", "AceConsole-3.0")
local DEBUG = false

---- BaikItemLevel Functions ----
function BaikItemLevel:DPrint(fmt, ...)
    if DEBUG then
        self:Print(string.format(fmt, ...))
    end
end

local DEFAULTS = {
    profile = {
        filterLevel = 150,
        colorLevelSimple = false,
        colorLevelPoor = -20,
        colorLevelCommon = -10,
        colorLevelUncommon = 0,
        colorLevelRare = 5,
        colorLevelEpic = 10,
        colorLevelLegend = 20,
    }
}

local optCounter = -1
local function nextOrder()
    optCounter = optCounter + 1
    return optCounter
end

local NUMBER_PATTERN = "(-?%d+)"

local function FilterItemLevel(value, prevBucket, nextBucket)
    local number = tonumber(string.match(value, NUMBER_PATTERN))

    if number < prevBucket then
        number = prevBucket + 1
    elseif number >= nextBucket then
        number = nextBucket - 1
    end

    return number
end

local MAX_ITEM_LEVELS = 10000

local function Db()
    return BaikItemLevel.db.profile
end

local options = {
    name = "BaikItemLevel",
    handler = BaikItemLevel,
    type = "group",
    args = {
        general = {
            name = "General Options",
            type = "group",
            childGroups = "select",
            args = {
                filter_header = {
                    name = "Item Level Filter",
                    type = "header",
                    order = nextOrder(),
                },
                filter_number = {
                    name = "Item Level",
                    type = "input",
                    pattern = NUMBER_PATTERN,
                    usage = "Default (150)",
                    order = nextOrder(),
                    get = function()
                        return tostring(Db().filterLevel)
                    end,
                    set = function(info, value)
                        local number = string.match(value, NUMBER_PATTERN)

                        Db().filterLevel = tonumber(number)
                        BaikItemLevel:Update()
                    end
                },
                color_range_header = {
                    name = "ItemLevel Color",
                    type = "header",
                    order = nextOrder(),
                },
                color_level_simple = {
                    name = "Simple (Use item quality)",
                    type = "toggle",
                    get = function()
                        return Db().colorLevelSimple
                    end,
                    set = function(info, value)
                        Db().colorLevelSimple = value
                        BaikItemLevel:Update()
                    end
                },
                color_level_group = {
                    name = "ItemLevel Range",
                    type = "group",
                    disabled = function()
                        return Db().colorLevelSimple
                    end,
                    args = {
                        color_level_poor = {
                            name = "Poor Level",
                            type = "input",
                            pattern = NUMBER_PATTERN,
                            usage = string.format("Default (%d)", DEFAULTS.profile.colorLevelPoor),
                            order = nextOrder(),
                            get = function()
                                return tostring(Db().colorLevelPoor)
                            end,
                            set = function(info, value)
                                local number = FilterItemLevel(value, -MAX_ITEM_LEVELS, Db().colorLevelCommon)

                                Db().colorLevelPoor = number
                                BaikItemLevel:Update()
                            end
                        },
                        color_level_common = {
                            name = "Common Level",
                            type = "input",
                            pattern = NUMBER_PATTERN,
                            usage = string.format("Default (%d)", DEFAULTS.profile.colorLevelCommon),
                            order = nextOrder(),
                            get = function()
                                return tostring(Db().colorLevelCommon)
                            end,
                            set = function(info, value)
                                local number = FilterItemLevel(value, Db().colorLevelPoor, Db().colorLevelUncommon)

                                Db().colorLevelCommon = number
                                BaikItemLevel:Update()
                            end
                        },
                        color_level_uncommon = {
                            name = "Uncommon Level",
                            type = "input",
                            pattern = NUMBER_PATTERN,
                            usage = string.format("Default (%d)", DEFAULTS.profile.colorLevelUncommon),
                            order = nextOrder(),
                            get = function()
                                return tostring(Db().colorLevelUncommon)
                            end,
                            set = function(info, value)
                                local number = FilterItemLevel(value, Db().colorLevelCommon, Db().colorLevelRare)

                                Db().colorLevelUncommon = tonumber(number)
                                BaikItemLevel:Update()
                            end
                        },
                        color_level_rare = {
                            name = "Rare Level",
                            type = "input",
                            pattern = NUMBER_PATTERN,
                            usage = string.format("Default (%d)", DEFAULTS.profile.colorLevelRare),
                            order = nextOrder(),
                            get = function()
                                return tostring(Db().colorLevelRare)
                            end,
                            set = function(info, value)
                                local number = FilterItemLevel(value, Db().colorLevelUncommon, Db().colorLevelEpic)

                                Db().colorLevelRare = tonumber(number)
                                BaikItemLevel:Update()
                            end
                        },
                        color_level_epic = {
                            name = "Epic Level",
                            type = "input",
                            pattern = NUMBER_PATTERN,
                            usage = string.format("Default (%d)", DEFAULTS.profile.colorLevelEpic),
                            order = nextOrder(),
                            get = function()
                                return tostring(Db().colorLevelEpic)
                            end,
                            set = function(info, value)
                                local number = FilterItemLevel(value, Db().colorLevelRare, Db().colorLevelLegend)

                                Db().colorLevelEpic = tonumber(number)
                                BaikItemLevel:Update()
                            end
                        },
                        color_level_legend = {
                            name = "Legend Level",
                            type = "input",
                            pattern = NUMBER_PATTERN,
                            usage = string.format("Default (%d)", DEFAULTS.profile.colorLevelLegend),
                            order = nextOrder(),
                            get = function()
                                return tostring(Db().colorLevelLegend)
                            end,
                            set = function(info, value)
                                local number = FilterItemLevel(value, Db().colorLevelEpic, MAX_ITEM_LEVELS)

                                Db().colorLevelLegend = tonumber(number)
                                BaikItemLevel:Update()
                            end
                        },
                    }
                },
            }
        }
    }
}

---- BaikItemLevel Functions ----
function BaikItemLevel:Update()
    for _, module in pairs(self.modules) do
        module:OnUpdate()
    end
end

---- Ace Callbacks ----
function BaikItemLevel:OnInitialize()
    self.db = LibStub("AceDB-3.0"):New("BaikItemLevelDB", DEFAULTS, true)
    self.options = options
    self.options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
    self.options.args.profiles["order"] = -1
    self.modules = {
        self:GetModule("Bag"),
        self:GetModule("Inspect"),
        self:GetModule("Inventory")
    }
    self:DPrint("OnInitialize")
end

function BaikItemLevel:OnEnable()
    LibStub("AceConfig-3.0"):RegisterOptionsTable("BaikItemLevel", self.options)
    LibStub("AceConfigDialog-3.0"):AddToBlizOptions("BaikItemLevel", "BaikItemLevel")
    self:DPrint("OnEnable")
end

function BaikItemLevel:OnDisable()
end
